#!/bin/bash
#
# Project code name "ORCA"
# 日医標準レセプトソフト(JMA standard receipt software)
#
# Copyright(C) 2002 JMA(Japan Medical Association)
#
# CREATE: 20161222
# UPDATE:
#
#
# データベース構造整合チェック処理
#
# Note

SCHEMAFILEP=orcadbschema-public.ver520
SCHEMAFILEM=orcadbschema-master.ver520
SCHEMAFILESP=orcadbschema-sum-public.ver520
SCHEMAFILESM=orcadbschema-sum-master.ver520
SCHEMAFILEVP=orcadbschema-view-public.ver520
SCHEMAMSG="現在のスキーマバージョンは5.2.0です。"

fin_message() {
  if [ ${ERROR} -eq 0 ]; then
    echo "--------------------------------------------------------" | tee -a ${LOGFILE}
    echo | tee -a ${LOGFILE}
    echo "  データベースの整合性チェックは正常に終了しました。    " | tee -a ${LOGFILE}
    echo | tee -a ${LOGFILE}
    echo "--------------------------------------------------------" | tee -a ${LOGFILE}
  else
    echo "--------------------------------------------------------" | tee -a ${LOGFILE}
    echo                                                            | tee -a ${LOGFILE}
    echo "  データベースの整合性に誤りがあります。                " | tee -a ${LOGFILE}
    echo "  詳細は$LOGFILEを参照ください。                        " | tee -a ${LOGFILE}
    echo                                                            | tee -a ${LOGFILE}
    echo "--------------------------------------------------------" | tee -a ${LOGFILE}
  fi
}

summary_check() {
  SQL="create table tbl as select tablename from pg_tables
  where schemaname = '${SCHEMANAME}' and tableowner = 'orca' order by tablename;
  \copy tbl to /tmp/table.txt
  drop table tbl;"
  echo "${SQL}" | sudo -u orca psql ${DBNAME} -Atq

  SQL="create table view as select viewname from pg_views
  where schemaname = '${SCHEMANAME}' and viewowner = 'orca' order by viewname;
  \copy view to /tmp/view.txt
  drop table view;"
  echo "${SQL}" | sudo -u orca psql ${DBNAME} -Atq

  SQL="create table idx as select a.indexname from pg_indexes a
  inner join pg_tables b on a.tablename = b.tablename
  and b.schemaname = '${SCHEMANAME}' and b.tableowner = 'orca'
  where a.schemaname = '${SCHEMANAME}' order by indexname;
  \copy idx to /tmp/index.txt
  drop table idx;"
  echo "${SQL}" | sudo -u orca psql ${DBNAME} -Atq

  cat /tmp/table.txt /tmp/view.txt /tmp/index.txt > /tmp/schema-summary

  SUMRES=`diff -u ${SCHEMAFILE} /tmp/schema-summary | grep '+[a-zA-Z]'`
  if [ -n "${SUMRES}" ]; then
    echo "" | tee -a ${LOGFILE}
    echo "日レセが提供していないスキーマがあります(${SCHEMANAME})。"  | tee -a ${LOGFILE}
    diff -u ${SCHEMAFILE} /tmp/schema-summary | grep '+[a-zA-Z]' >> ${LOGFILE}
    echo ""  >> ${LOGFILE}
    ERROR=1
  fi

  rm /tmp/table.txt
  rm /tmp/view.txt
  rm /tmp/index.txt
  rm /tmp/schema-summary
}

view_check() {
  SQL="create table view as select viewname,definition from pg_views
  where schemaname = '${SCHEMANAME}' and viewowner = 'orca' order by viewname;
  \copy view to /tmp/view.txt
  drop table view;"
  echo "${SQL}" | sudo -u orca psql ${DBNAME} -Atq

  SUMRES=`diff -u ${SCHEMAFILE} /tmp/view.txt | grep '+[a-zA-Z]'`
  if [ -n "${SUMRES}" ]; then
    echo "" | tee -a ${LOGFILE}
    echo "ビューで不整合があります(${SCHEMANAME})。"  | tee -a ${LOGFILE}
    diff -u ${SCHEMAFILE} /tmp/view.txt | grep '+[a-zA-Z]' >> ${LOGFILE}
    echo ""  >> ${LOGFILE}
    ERROR=1
  fi

  rm /tmp/view.txt
}


schema_check() {
for list in `ruby -e "puts STDIN.read.gsub(/(.*?)\|.*/,'\1')"  < ${SCHEMAFILE} | uniq` ; do
  kind=${list:0:1}
  schema=${list:2}
  # echo "-- kind [${kind}] ${schema} --"

# 一時テーブルチェック
#if [ ${PGVER} = "10.8" ] || \
#   [ ${PGVER} = "10.7" ] || \
#   [ ${PGVER} = "10.6" ] || \
#   [ ${PGVER} = "9.5" ]; then
  SQLOPT="c.relpersistence <> 't'"
#else
#  SQLOPT="c.relistemp = false"
#fi

SQLORG="
SELECT
    c.relkind || ':' || c.relname,
    a.attname,
    format_type(a.atttypid, a.atttypmod) as atttypedefni,
    rtrim(pg_get_expr(d.adbin,d.adrelid),'::bpchar') as default,
    a.attnotnull
FROM
    pg_attribute a
        LEFT OUTER JOIN pg_type t ON a.atttypid = t.oid
        LEFT OUTER JOIN pg_class c on a.attrelid = c.oid
        LEFT OUTER JOIN pg_attrdef d on d.adrelid = c.oid and d.adnum = a.attnum
WHERE
    c.relname = '${schema}'
    AND c.relkind = '${kind}'
    AND a.attnum > 0
    AND attisdropped = false
    AND ${SQLOPT}
ORDER BY
    c.relname,a.attnum ;"

  echo "${SQLORG}" | sudo -u orca psql -Aqt ${DBNAME} > ${CHECKFILE}
  if [ -s ${CHECKFILE} ]; then
    grep -E "^${kind}:${schema}\|" ${SCHEMAFILE} | diff -u -  ${CHECKFILE} > /dev/null 2>&1
    if [ $? -eq 0 ]; then
       echo -e "${schema}\c"
       echo -e "                                     \r\c"
    else
       echo -e "                                     \r\c"
       echo ""
       echo "${SCHEMANAME}.${schema} が不整合です。"  | tee -a ${LOGFILE}
       grep -E "^${kind}:${schema}\|" ${SCHEMAFILE} | diff -u -  ${CHECKFILE} >> ${LOGFILE} 2>&1
       echo ""  >> ${LOGFILE}
       ERROR=1
    fi
  else
    if [ "${SCHEMANAME}.${schema}" != "public.montsuqi_auditlog" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.montsuqi_auditlog_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_pgid" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_start" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_clog" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_log" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_clog_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_log_pgid" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_log_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monbatch_log_start" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent_event" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent_pushed_at" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent_user" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpushevent_uuid" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.ap_server_ld_processes" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.aps_migrations" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpassword" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.ap_server_ld_processes_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.index_ap_server_ld_processes_on_ld" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monpassword_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monblob_importtime" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monblob_pkey" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.monblob_blobid" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_adrs_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_chk_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_hknjainf_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_mstkanri_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_srycdchg_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_tensu_org" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_adrs_org_post" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_srycdchg_org_rjnsrycd" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_tensu_org_formalname" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_tensu_org_kananame" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_tensu_org_kensagrp" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_tensu_org_name" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.idx_tensu_org_yakkakjncd" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_adrs_org_primary_key" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_chk_org_primary_key" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_hknjainf_org_primary_key" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_mstkanri_org_primary_key" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_srycdchg_org_primary_key" ] && \
       [ "${SCHEMANAME}.${schema}" != "public.tbl_tensu_org_primary_key" ]; then
       echo -e "                                     \r\c"
       echo ""
       echo "${SCHEMANAME}.${schema} が存在しません。" | tee -a ${LOGFILE}
       ERROR=1
    fi
  fi
done
echo -e "                                     \r\c"
}

# Main

CHECKFILE=orcadbcheck.txt
ERROR=0

if [ -e ${LOGFILE} ]; then
  rm -f ${LOGFILE}
fi

PGVER=`sudo -u orca psql ${DBNAME} -Atc "select substring(version(),12,4);"`
if [ ${PGVER:0:2} = "9." ]; then
  PGVER=${PGVER:0:3}
fi
VERSION=`sudo -u orca psql ${DBNAME} -Atc "select trim(version) from tbl_dbkanri where kanricd = 'ORCADB00';"`
echo
echo "ORCA Project Jma-Receipt Database Schema Check (${TOOLVER})" | tee -a ${LOGFILE}
echo "処理日:[ $(date) ]" >> ${LOGFILE}
echo "== Operating System information ==" | tee -a ${LOGFILE}
UNAME=`uname -a`
echo "  ${UNAME}" | tee -a ${LOGFILE}
if [ -e /etc/lsb-release ]; then
  sed 's/^/  /' /etc/lsb-release | tee -a ${LOGFILE}
else
  echo "  /etc/lsb-release is not found." | tee -a ${LOGFILE}
fi
echo "== Database System information ==" | tee -a ${LOGFILE}
echo "  PostgreSQL version ${PGVER}" | tee -a ${LOGFILE}
echo "  Database Name : ${DBNAME}" | tee -a ${LOGFILE}
echo "== Package information ==" | tee -a ${LOGFILE}
if [ -e /opt/jma/weborca ]; then
  PACKAGE=`dpkg -l jma-receipt-weborca | grep jma-receipt-weborca`
  echo "  ${PACKAGE}" | tee -a ${LOGFILE}
  if [ -e /opt/jma/weborca/app/doc/version ]; then
    JRVER=`grep version /opt/jma/weborca/app/doc/version`
    echo "  ${JRVER}" | tee -a ${LOGFILE}
  fi
else
  PACKAGE=`dpkg -l jma-receipt | grep jma-receipt`
  echo "  ${PACKAGE}" | tee -a ${LOGFILE}
  if [ -e /usr/lib/jma-receipt/doc/version ]; then
    JRVER=`grep version /usr/lib/jma-receipt/doc/version`
    echo "  ${JRVER}" | tee -a ${LOGFILE}
  fi 
  PACKAGE=`dpkg -l panda-server | grep panda-server`
  echo "  ${PACKAGE}" | tee -a ${LOGFILE}
fi
echo "== Schema Check information ==" | tee -a ${LOGFILE}
echo "  Jma-Receipt Schema Version (${VERSION})" | tee -a ${LOGFILE}

if [ ! -e ${SCHEMAFILESP} ]; then
  echo "チェックファイル ${SCHEMAFILESP} がありません。"
  exit 1
fi
if [ ! -e ${SCHEMAFILESM} ]; then
  echo "チェックファイル ${SCHEMAFILESM} がありません。"
  exit 1
fi
if [ ! -e ${SCHEMAFILEP} ]; then
  echo "チェックファイル ${SCHEMAFILEP} がありません。"
  exit 1
fi
if [ ! -e ${SCHEMAFILEM} ]; then
  echo "チェックファイル ${SCHEMAFILEM} がありません。"
  exit 1
fi
if [ ! -e ${SCHEMAFILEVP} ]; then
  echo "チェックファイル ${SCHEMAFILEVP} がありません。"
  exit 1
fi

FILEINFO=`stat -c %y ${SCHEMAFILESP} | cut -b 1-19`
echo "  チェックファイル(public):[ ${SCHEMAFILESP} : ${FILEINFO} ]" | tee -a ${LOGFILE}
FILEINFO=`stat -c %y ${SCHEMAFILESM} | cut -b 1-19`
echo "  チェックファイル(master):[ ${SCHEMAFILESM} : ${FILEINFO} ]" | tee -a ${LOGFILE}
FILEINFO=`stat -c %y ${SCHEMAFILEP} | cut -b 1-19`
echo "  チェックファイル(public):[ ${SCHEMAFILEP} : ${FILEINFO} ]" | tee -a ${LOGFILE}
FILEINFO=`stat -c %y ${SCHEMAFILEM} | cut -b 1-19`
echo "  チェックファイル(master):[ ${SCHEMAFILEM} : ${FILEINFO} ]" | tee -a ${LOGFILE}
FILEINFO=`stat -c %y ${SCHEMAFILEVP} | cut -b 1-19`
echo "  チェックファイル(public):[ ${SCHEMAFILEVP} : ${FILEINFO} ]" | tee -a ${LOGFILE}

if [ -e ${CHECKFILE} ]; then
  rm -f ${CHECKFILE}
fi

echo
echo ${SCHEMAMSG}
echo "スキーマの整合性をチェックします。"

# master スキーマの存在チェック
SCHEMA=`sudo -u orca psql ${DBNAME} -Atc "select schema_name from information_schema.schemata where schema_name='master' and schema_owner='orca';"`

if ! [ "${SCHEMA}" = "master" ]; then
  echo ""
  echo "master スキーマが存在しません。" | tee -a ${LOGFILE}
  ERROR=1
  fin_message
  exit 0
fi

# 日レセ以外のorcaオーナーのスキーマをチェック(publicスキーマ内)
SCHEMAFILE=${SCHEMAFILESP}
SCHEMANAME="public"
summary_check

# 日レセ以外のorcaオーナーのスキーマをチェック(masterスキーマ内)
SCHEMAFILE=${SCHEMAFILESM}
SCHEMANAME="master"
summary_check

# 日レセのスキーマをチェック
# スキーマ(public)
SCHEMAFILE=${SCHEMAFILEP}
SCHEMANAME="public"
schema_check

# スキーマ(master)
SCHEMAFILE=${SCHEMAFILEM}
SCHEMANAME="master"
schema_check

# ビュー(public)
SCHEMAFILE=${SCHEMAFILEVP}
SCHEMANAME="public"
view_check

if [ -e ${CHECKFILE} ]; then
  rm -f ${CHECKFILE}
fi

echo
fin_message

exit 0
