#!/bin/bash
#
# Project code name "ORCA"
# 日医標準レセプトソフト(JMA standard receipt software)
#
# Copyright(C) 2002 JMA(Japan Medical Association)
#
# CREATE: 20071010
# UPDATE: 20071213 実行ユーザーチェック追加,文字コード対応
#
#
# データベース構造整合チェック処理
#
# Note
#  2018. 9. 6 TOOLVER=2.1.0にした　スクリプトを修正した場合はマイナー以上
#             マッチングデータを修正した場合はメンテナンスをカウントアップ
#  2016.12.22 ver5.0系対応のためバージョン別のディレクトリ構成にする
#  2014. 7. 2 ＯＳ、日レセ、スキーマチェックのバージョン情報を出力
#  2013. 5.29 一時テーブルを除外する対処 
#  2010. 8.16 Lucid 対応 
#  2009. 8.31 view_q002 8.1のダンプを8.3でリストアした場合不整合が発生する対処
#             orcadbschema.ver440.8.3.exception 作成
#             キーワード:EXCEPTION exception 

JMARECEIPT_ENV="/etc/jma-receipt/jma-receipt.env"
if [ ! -f ${JMARECEIPT_ENV} ]; then
    echo "${JMARECEIPT_ENV} does not found."
    exit 1
fi
. $JMARECEIPT_ENV

FLAG=`id -u`

if [ "${LANG}" != "ja_JP.UTF-8" ] && [ "${LANG}" != "ja_JP.utf8" ]; then
  echo "ERROR: change charactor-code ja_JP.UTF-8"
  exit 1
fi

if [ ${FLAG} -ne 0 ]; then
  echo "ERROR: root権限で実行してください。"
  exit 1
fi

USER=`stat -c %U $0`

export TOOLVER="4.4.0"
export DBNAME
export LOGFILE=jma-receipt-dbscmchk.log

VERSION=`sudo -u orca psql ${DBNAME} -t -A -c "select trim(version) from tbl_dbkanri where kanricd = 'ORCADB00';"`

if   [ "${VERSION}" = "050200-1" ]; then
  VER="5.2.0"
elif   [ "${VERSION}" = "050100-1" ]; then
  VER="5.1.0"
else
  echo "データベーススキーマのバージョンが正しくありません。"
  echo ${VERSION}
  exit 1
fi

cd ${VER}
bash jma-receipt-dbscmchk-${VER}.sh
cd ..

cp -p ${VER}/${LOGFILE} .
rm ${VER}/${LOGFILE}

if [ -e $LOGFILE ]; then
  chown $USER:$USER $LOGFILE
  chmod 644 $LOGFILE
fi

exit 0
